<?php
/**
 * @package   OSDownloads-Pro
 * @contact   www.joomlashack.com, help@joomlashack.com
 * @copyright 2019-2020 Joomlashack.com. All rights reserved
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 *
 * This file is part of OSDownloads-Pro.
 *
 * OSDownloads-Pro is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * OSDownloads-Pro is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSDownloads-Pro.  If not, see <http://www.gnu.org/licenses/>.
 */

use Alledia\OSDownloads\Pro\MailingList\MailChimp;
use Illuminate\Support\Collection;
use Joomla\CMS\Application\CMSApplication;

defined('_JEXEC') or die();

require_once __DIR__ . '/MailinglistFieldTraits.php';

JFormHelper::loadFieldClass('list');

abstract class AbstractMailChimp extends JFormFieldList
{
    use MailinglistFieldTraits;

    /**
     * @var CMSApplication
     */
    protected $app = null;

    /**
     * The message to display if wasn't able to render the input
     *
     * @var string
     */
    protected $errorMessageForInput = '';

    /**
     * @var int
     */
    protected $requestCount = 100;

    /**
     * @var array[]object[]
     */
    protected static $mailchimpResponses = array();

    /**
     * @param SimpleXMLElement $element
     * @param mixed            $value
     * @param null             $group
     *
     * @return bool
     * @throws Exception
     */
    public function setup(SimpleXMLElement $element, $value, $group = null)
    {
        $success = $this->isMailinglistEnabled('mailchimp')
            && parent::setup($element, $value, $group);

        if ($success && !defined('OSDOWNLOADS_LOADED')) {
            $includePath = JPATH_ADMINISTRATOR . '/components/com_osdownloads/include.php';
            if ($success = is_file($includePath)) {
                require_once $includePath;
            }
        }

        $this->app = JFactory::getApplication();

        return $success;
    }

    /**
     * @return string
     * @throws Exception
     */
    protected function getInput()
    {
        if ($upgradePhp = MailChimp::getPhpUpgradeMessage()) {
            $this->setError($upgradePhp);
        }

        $output = parent::getInput();
        if (empty($this->errorMessageForInput)) {
            return $output;
        }

        return '<em>' . $this->errorMessageForInput . '</em>';
    }

    /**
     * @return object[]
     */
    protected function getOptions()
    {
        return array_merge(parent::getOptions(), $this->getMailchimpOptions());
    }

    /**
     * @return object[]
     */
    abstract protected function getMailchimpOptions();

    /**
     * @param string $endpoint
     * @param array  $options
     *
     * @return object[]
     * @throws Exception
     */
    protected function mailchimpRequest($endpoint, array $options = null)
    {
        $key = md5(serialize(func_get_args()));

        if (!isset(static::$mailchimpResponses[$key])) {
            static::$mailchimpResponses[$key] = false;

            try {
                if ($mc = MailChimp::getMailChimp()) {
                    $response = $mc->request($endpoint, $options);

                    if ($response instanceof Collection) {
                        $response = $response->toArray();
                    }

                    static::$mailchimpResponses[$key] = $response;

                } else {
                    if ($this->form->getName() == 'com_config.component') {
                        JHtml::_('behavior.tooltip');

                        $message = sprintf(
                            '<span>%s <i class="icon-info hasTip" title="%s"></i></span>',
                            JText::_('COM_OSDOWNLOADS_ERROR_INVALID_API_AUTH'),
                            JText::_('COM_OSDOWNLOADS_ML_MAILCHIMP_INVALID_API_TOOLTIP')
                        );

                    } else {
                        $message = JText::_('COM_OSDOWNLOADS_ERROR_INVALID_API_AUTH');
                    }

                    throw new Exception($message, 500);
                }

            } catch (Exception $error) {
                // Check for errors received from Mailchimp
                $errorMessage = json_decode($error->getMessage());
                if (!$errorMessage) {
                    $errorMessage = (object)array(
                        'detail' => $error->getMessage() ?: 'Unknown',
                        'status' => $error->getCode() ?: 500
                    );
                }

                // Not found errors are okay
                if ($errorMessage->status != 404) {
                    throw new Exception($errorMessage->detail, $errorMessage->status);
                }
            }
        }

        return static::$mailchimpResponses[$key] ?: null;
    }

    /**
     * @return object[]
     * @throws Exception
     */
    protected function getMailchimpLists()
    {
        $lists = $this->mailchimpRequest(
            'lists',
            array(
                'fields' => 'lists.id,lists.name',
                'count'  => $this->requestCount
            )
        );

        return $lists;
    }

    /**
     * @param string $listId
     *
     * @return object[]
     * @throws Exception
     */
    protected function getMailchimpGroups($listId)
    {
        $groups = array();

        $interestCategories = (array)$this->mailchimpRequest(
            sprintf('lists/%s/interest-categories', $listId),
            array(
                'fields' => 'categories.id,categories.title',
                'count'  => $this->requestCount
            )
        );

        foreach ($interestCategories as $category) {
            $interests = $this->mailchimpRequest(
                sprintf('lists/%s/interest-categories/%s/interests', $listId, $category->id),
                array(
                    'fields' => 'interests.id,interests.name',
                    'count'  => $this->requestCount
                )
            );

            foreach ($interests as $interest) {
                $groups[] = (object)array(
                    'category' => $category->title,
                    'interest' => $interest->name,
                    'id'       => $interest->id
                );
            }
        }

        return $groups;
    }

    /**
     * @param string $message
     * @param string $type
     *
     * @return void
     */
    protected function setError($message, $type = null)
    {
        if ($type !== null) {
            $this->errorMessageForInput = JText::_('COM_OSDOWNLOADS_ERROR');
            $this->app->enqueueMessage($message, $type);

        } else {
            $this->errorMessageForInput = $message;
        }
    }
}
