<?php
/**
 * @package   OSDownloads-Pro
 * @contact   www.joomlashack.com, help@joomlashack.com
 * @copyright 2005-2020 Joomlashack.com. All rights reserved
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 *
 * This file is part of OSDownloads-Pro.
 *
 * OSDownloads-Pro is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * OSDownloads-Pro is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSDownloads-Pro.  If not, see <http://www.gnu.org/licenses/>.
 */

use Ctct\ConstantContact;
use Ctct\Exceptions\CtctException;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;

defined('_JEXEC') or die();

JFormHelper::loadFieldClass('list');

require_once __DIR__ . '/MailinglistFieldTraits.php';

class OsdownloadsFormFieldConstantContactList extends JFormFieldList
{
    use MailinglistFieldTraits;

    protected $type = 'osdownloads.constantcontactlist';

    /**
     * @var object[]
     */
    protected $ccOptions = null;

    /**
     * @var string
     */
    protected $errorMessage = null;

    /**
     * @var string
     */
    protected $errorDisplay = '<span class="alert alert-error">%s</span>';

    /**
     * @inheritDoc
     */
    public function setup(SimpleXMLElement $element, $value, $group = null)
    {
        $success = $this->isMailinglistEnabled('cc')
            && $success = parent::setup($element, $value, $group);

        if ($success && !defined('OSDOWNLOADS_LOADED')) {
            $includePath = JPATH_ADMINISTRATOR . '/components/com_osdownloads/include.php';
            if ($success = is_file($includePath)) {
                require_once $includePath;
            }
        }

        $ccPath = JPATH_ADMINISTRATOR . '/components/com_osdownloads/library/Ctct/autoload.php';

        $success = $success && is_file($ccPath);
        if ($success) {
            require_once $ccPath;
        }

        return $success;
    }

    /**
     * @return object[]
     */
    protected function getCCOptions()
    {
        if ($this->ccOptions === null) {
            $this->ccOptions = [];

            $params      = JComponentHelper::getParams('com_osdownloads');
            $apiKey      = $params->get('mailinglist.cc.apikey', '');
            $accessToken = $params->get('mailinglist.cc.access_token', '');

            if (!$apiKey || !$accessToken) {
                $this->errorMessage = Text::_('COM_OSDOWNLOADS_ERROR_INVALID_API_AUTH');

            } else {
                try {
                    $cc = new ConstantContact($apiKey);

                    $lists = $cc->listService->getLists($accessToken);
                    foreach ($lists as $list) {
                        $this->ccOptions[] = HTMLHelper::_('select.option', $list->id, $list->name);
                    }

                } catch (CtctException $ccError) {
                    $errors = $ccError->getErrors();
                    $error  = array_shift($errors);

                    $this->errorMessage = empty($error['error_message'])
                        ? 'Unknown Constant Contact Error'
                        : $error['error_message'];

                } catch (Exception $error) {
                    $this->errorMessage = $error->getMessage();

                } catch (Throwable $error) {
                    $this->errorMessage = $error->getMessage();
                }
            }
        }

        return $this->ccOptions;
    }

    /**
     * @inheritDoc
     */
    protected function getInput()
    {
        $ccLists = $this->getCCOptions();

        if ($this->errorMessage) {
            return sprintf($this->errorDisplay, $this->errorMessage);

        } elseif (!$ccLists) {
            return sprintf($this->errorDisplay, Text::_('COM_OSDOWNLOADS_ML_CONSTANTCONTACT_NO_LISTS'));
        }

        return parent::getInput();
    }

    /**
     * @inheritDoc
     */
    protected function getOptions()
    {
        $options = array_merge(parent::getOptions(), $this->getCCOptions());

        $defaultOptionLabel = (string)$this->element['empty_label'] ?: 'COM_OSDOWNLOADS_SELECT_LIST';
        array_unshift($options, HTMLHelper::_('select.option', 0, Text::_($defaultOptionLabel)));

        return $options;
    }
}
