<?php
/**
 * @package   OSDownloads-Pro
 * @contact   www.joomlashack.com, help@joomlashack.com
 * @copyright 2005-2020 Joomlashack.com. All rights reserved
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 *
 * This file is part of OSDownloads-Pro.
 *
 * OSDownloads-Pro is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * OSDownloads-Pro is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSDownloads-Pro.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Alledia\OSDownloads\Pro\Joomla\Plugin;

use Alledia\OSDownloads\Free\Factory;
use Alledia\OSDownloads\Free\Joomla;
use Alledia\OSDownloads\Free\DisplayData;
use Exception;
use JEventDispatcher;
use JForm;
use JHtml;
use JLayoutHelper;
use Joomla\Registry\Registry;
use JText;
use Throwable;

defined('_JEXEC') or die();

class Content extends \Alledia\OSDownloads\Free\Joomla\Plugin\Content
{
    /**
     * @var null
     */
    protected $client = null;

    /**
     * @var bool
     */
    protected $supportLoaded = false;

    /**
     * @var bool
     */
    protected $tagsFound = false;

    /**
     * @var Joomla\Component\Site
     */
    protected $component = null;

    /**
     * PlgContentOSDownloads constructor.
     *
     * @param JEventDispatcher $subject
     * @param array            $config
     *
     * @throws Exception
     */
    public function __construct($subject, array $config)
    {
        parent::__construct($subject, $config);

        $this->client = Factory::getApplication()->client;

        if ($this->isEnabled()
            && Factory::getUser()->guest
        ) {
            // Always load support files if under aggressive caching
            $cache = Factory::getCache();
            if (isset($cache->options['caching']) && $cache->options['caching']) {
                $this->loadSupport();
            }
        }
    }

    /**
     * @param string   $context
     * @param object   $item
     * @param Registry $params
     *
     * @return void
     */
    public function onContentBeforeDisplay($context, $item, $params)
    {
        if ($this->enabled
            && !empty($item->text)
            && $this->checkTagsFound($item->text)
        ) {
            try {
                $this->loadSupport();
                $this->replaceTags($item->text);

            } catch (Exception $e) {
                $this->app->enqueueMessage(
                    JText::sprintf('COM_OSDOWNLOADS_ERROR_PLUGIN', __CLASS__, $e->getMessage()),
                    'error'
                );

            } catch (Throwable $e) {
                $this->app->enqueueMessage(
                    JText::sprintf('COM_OSDOWNLOADS_ERROR_PLUGIN', __CLASS__, $e->getMessage()),
                    'error'
                );
            }
        }
    }

    /**
     * @param JForm  $form
     * @param object $data
     *
     * @return bool
     * @throws Exception
     */
    public function onContentPrepareForm($form, $data)
    {
        // Update the form with pro features before other plugins get a chance
        if ($this->app->isClient('administrator')) {
            switch ($form->getName()) {
                case 'com_categories.categorycom_osdownloads':
                    $groupsPlaceholder = JText::_('JGLOBAL_USE_GLOBAL');
                    break;

                case 'com_osdownloads.file':
                    $proForm = JPATH_ADMINISTRATOR . '/components/com_osdownloads/models/forms/file_pro.xml';
                    if (is_file($proForm)) {
                        $form->loadFile($proForm, true);

                        $groupsPlaceholder = JText::_('COM_OSDOWNLOADS_INHERIT_FROM_CATEGORY');
                    }
                    break;
            }

            if (!empty($groupsPlaceholder)) {
                JHtml::_(
                    'formbehavior.chosen',
                    '#jform_params_mailinglist_mailchimp_groups',
                    null,
                    array(
                        'placeholder_text_multiple' => $groupsPlaceholder
                    )
                );
            }
        }

        return parent::onContentPrepareForm($form, $data);
    }

    /**
     * @param string   $context
     * @param object   $item
     * @param Registry $params
     *
     * @return void
     * @throws Exception
     */
    public function onContentPrepare($context, $item, $params)
    {
        $this->onContentBeforeDisplay($context, $item, $params);
    }

    /**
     * Load all support css/js required to handle the popup
     * NB: If needed, this must be called no later than onBeforeRender()
     *
     * @return void
     * @throws Exception
     */
    protected function loadSupport()
    {
        if (!$this->supportLoaded) {
            $this->supportLoaded = true;

            // Load jQuery?
            if (Factory::getApplication()->get('jquery') !== true
            ) {
                Factory::getApplication()->set('jquery', true);
                JHtml::_('jquery.framework');
            }

            JHtml::_('behavior.formvalidation');

            $options = array(
                'version'  => $this->getComponent()->getMediaVersion(),
                'relative' => true
            );

            JHtml::_('script', 'com_osdownloads/jquery.osdownloads.bundle.min.js', $options, array());
            JHtml::_('stylesheet', 'com_osdownloads/frontend.css', $options, array());
        }
    }

    /**
     * @param string $text
     *
     * @return bool
     */
    protected function checkTagsFound($text)
    {
        if ($this->enabled) {
            $this->tagsFound = $this->tagsFound
                || (is_string($text) && utf8_strpos($text, '{osdownloads ') !== false);

            return $this->tagsFound;
        }

        return false;
    }

    /**
     * Look for recognized shortcodes in the text and replace.
     * If you have more than one download button for the same file, it is
     * advised to have different tags in the content for each one. The
     * easier way is to force a custom class for each button. So both won't
     * be replaced at the same time, keeping different unique ID for each.
     *
     * @param string $text
     *
     * @return bool
     * @throws Exception
     */
    protected function replaceTags(&$text)
    {
        if (!$this->checkTagsFound($text)) {
            return false;
        }

        $model = $this->getModel();

        // {osdownloads download_button[.css_class.css_class2] 3 "link text"}
        $tagRegex = array(
            '#{osdownloads[\s]+download_button([^}\s\n\t]+)?\s([0-9a-z\-_]+)(?:[\s\t]+"([^"]+)")?}#i'
        );

        $tagsReplaced = false;
        foreach ($tagRegex as $regex) {
            if (preg_match_all($regex, $text, $matches)) {
                $tagsReplaced = true;
                foreach ($matches[0] as $k => $source) {
                    $buttonClasses = trim(implode(' ', explode('.', $matches[1][$k])));
                    $fileId        = (int)$matches[2][$k];
                    $buttonLabel   = preg_replace('/[\'"]/', '', $matches[3][$k]);

                    if (empty($fileId)) {
                        continue;
                    }

                    $file = $model->getItem($fileId);

                    if (empty($file)) {
                        $text = str_replace(
                            $source,
                            JText::sprintf('COM_OSDOWNLOADS_ERROR_FILE_NOT_FOUND', $fileId, 'getFileFromId'),
                            $text
                        );

                    } else {
                        /*
                         * We need to allow each instance of the shortcode
                         * to generate its own unique html due to unique ids
                         * required for each instance.
                         */
                        $data                = new DisplayData();
                        $data->item          = $file;
                        $data->params        = new Registry();
                        $data->buttonClasses = $buttonClasses;

                        $data->item->download_text = $buttonLabel;

                        $sourceLength = strlen($source);
                        while (($pos = strpos($text, $source)) !== false) {
                            $replacement = sprintf(
                                '<div class="osdownloadsaction"><div class="btn_download">%s</div></div>',
                                JLayoutHelper::render(
                                    'buttons.download',
                                    $data,
                                    null,
                                    array('component' => 'com_osdownloads')
                                )
                            );

                            $text = substr_replace($text, $replacement, $pos, $sourceLength);
                        }
                    }
                }
            }
        }

        return $tagsReplaced;
    }

    /**
     * @return Joomla\Component\Site|Object
     */
    protected function getComponent()
    {
        if ($this->component === null) {
            $this->component = Joomla\Component\Site::getInstance();
            $this->component->loadLibrary();
        }

        return $this->component;
    }

    /**
     * @param string $name
     *
     * @return \Alledia\OSDownloads\Pro\Joomla\Model\Item
     */
    protected function getModel($name = 'Item')
    {
        $model = $this->getComponent()->getModel($name);

        return $model;
    }
}
