<?php
/**
 * @package   OSDownloads-Pro
 * @contact   www.joomlashack.com, help@joomlashack.com
 * @copyright 2005-2020 Joomlashack.com. All rights reserved
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 *
 * This file is part of OSDownloads-Pro.
 *
 * OSDownloads-Pro is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * OSDownloads-Pro is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSDownloads-Pro.  If not, see <http://www.gnu.org/licenses/>.
 */

use Joomla\Registry\Registry;

defined('_JEXEC') or die;

if (file_exists(__DIR__ . '/admin/abstract.script.installer.php')) {
    require_once __DIR__ . '/admin/abstract.script.installer.php';
} else {
    require_once __DIR__ . '/abstract.script.installer.php';
}

class Com_OSDownloadsInstallerScript extends AbstractOSDownloadsInstallerScript
{
    /**
     * @var \Mailchimp\Mailchimp
     */
    protected $mailchimpInstance = null;

    public function preFlight($type, $parent)
    {
        // Special case for previous minimum version due to database management changes
        if ($type == 'update') {
            $previousVariant = $this->previousManifest->variant;
            $previousVersion = $this->previousManifest->version;
            if ($previousVariant == 'FREE' && version_compare($previousVersion, '1.13.1', 'lt')) {
                $updateLink = JRoute::_('index.php?option=com_installer&view=update');
                $this->setMessage(
                    JText::sprintf('COM_OSDOWNLOADS_ERROR_UPDATE_FREE_INSTALLATION', $updateLink),
                    'error'
                );
                $this->showMessages();
                return false;
            }
        }

        return parent::preFlight($type, $parent);
    }

    /**
     * Method to run after an install/update method
     *
     * @param string            $type
     * @param JInstallerAdapter $parent
     *
     * @return void
     * @throws Exception
     */
    public function postFlight($type, $parent)
    {
        try {
            parent::postFlight($type, $parent);

            $this->fixDocumentTableStructure();
            $this->fixCategoryParams();
            $this->fixCategoryViewParams();
            $this->setDefaultParams();

        } catch (Exception $e) {
            $this->setMessage($e->getMessage(), 'error');

        } catch (Throwable $e) {
            $this->setMessage($e->getMessage(), 'error');
        }

        $this->showMessages();
    }

    /**
     * Update component default parameters as needed
     */
    protected function setDefaultParams()
    {
        /** @var JTableExtension $table */
        $table = JTable::getInstance('Extension');
        $table->load(array('element' => 'com_osdownloads', 'type' => 'component'));

        $params = new Registry($table->params);
        $values = $params->toObject();

        if (!property_exists($values, 'email_confirmation_body')) {
            $params->set('email_confirmation_body', JText::_('COM_OSDOWNLOADS_EMAIL_CONFIRMATION_BODY_DEFAULT'));
            $table->params = $params->toString();
            $table->store();
        }
    }

    /**
     * Update category parameters for change as of v1.13.0
     *
     * @return void
     */
    protected function fixCategoryParams()
    {
        $db = JFactory::getDbo();

        $paramMap = array(
            'mailchimp_list'       => 'mailinglist.mailchimp.list_id',
            'mailchimp_groups'     => 'mailinglist.mailchimp.groups',
            'constantcontact_list' => 'mailinglist.cc.list_id'
        );

        $paramSearch = array();
        foreach ($paramMap as $key => $value) {
            $paramSearch[] = 'params LIKE ' . $db->quote('%' . $key . '%');
        }

        $query = $db->getQuery(true)
            ->select(
                array(
                    'id',
                    'params'
                )
            )
            ->from('#__categories')
            ->where(
                array(
                    'extension = ' . $db->quote('com_osdownloads'),
                    sprintf('(%s)', join(' OR ', $paramSearch))
                )
            );

        $categories = $db->setQuery($query)->loadObjectList();

        foreach ($categories as $category) {
            $params = new \Joomla\Registry\Registry($category->params);

            foreach ($paramMap as $oldKey => $newKey) {
                $params->set($newKey, $params->get($oldKey));
                $params->set($oldKey, null);
            }
            $paramsArray = $params->toArray();
            $newParams   = new \Joomla\Registry\Registry(array_filter($paramsArray));

            $category->params = $newParams->toString();

            $db->updateObject('#__categories', $category, 'id');
        }
    }

    /**
     * Created/executed via parent class
     *
     * @return array
     */
    protected function getParameterChangeMap()
    {
        $parameterMap = array_merge(
            parent::getParameterChangeMap(),
            array(
                'mailchimp_groups'                 => 'mailinglist.mailchimp.groups',
                'mailchimp_double_option'          => 'mailinglist.mailchimp.double_optin',
                'connect_constantcontact'          => 'mailinglist.cc.enable',
                'constantcontact_api_key'          => 'mailinglist.cc.apikey',
                'constantcontact_api_access_token' => 'mailinglist.cc.access_token',
                'constantcontact_default_list_id'  => 'mailinglist.cc.list_id',
                'constantcontact_default_status'   => 'mailinglist.cc.status'
            )
        );

        return $parameterMap;
    }

    /**
     * Copies the data from the pre Schema 1.10.0 document table
     * and drops the old fields
     *
     * @Since v1.13.0
     */
    public function fixDocumentTableStructure()
    {
        $db = JFactory::getDbo();

        $documentSchema = $db->getTableColumns('#__osdownloads_documents');

        $paramMap = array(
            'mailchimp_list'       => 'mailinglist.mailchimp.list_id',
            'mailchimp_groups'     => 'mailinglist.mailchimp.groups',
            'constantcontact_list' => 'mailinglist.cc.list_id',
        );

        if ($existingFields = array_intersect_key($paramMap, $documentSchema)) {
            if (count($existingFields) != count($paramMap)) {
                $this->setMessage(JText::_('COM_OSDOWNLOADS_ERROR_UPDATE_DOCUMENT_MISMATCH', 'error'));
                return;
            }

            $query = $db->getQuery(true)
                ->select(
                    array_merge(
                        array(
                            'id',
                            'params'
                        ),
                        array_keys($existingFields)
                    )
                )
                ->from('#__osdownloads_documents');

            $documents = $db->setQuery($query)->loadObjectList();
            foreach ($documents as $document) {
                $params = new \Joomla\Registry\Registry($document->params);
                foreach ($paramMap as $fieldName => $key) {
                    $value = $document->$fieldName;
                    if ($fieldName == 'mailchimp_groups') {
                        $value = json_decode($value);
                    }
                    $params->set($key, $value);
                }
                $document->params = $params->toObject();
                $updateQuery      = $db->getQuery(true)
                    ->update('#__osdownloads_documents')
                    ->set(
                        array(
                            'params = ' . $db->quote($params->toString())
                        )
                    )
                    ->where('id = ' . $document->id);
                $db->setQuery($updateQuery)->execute();
            }

            $fields = array_map(
                function ($field) use ($db) {
                    return 'DROP COLUMN ' . $db->quoteName($field);
                },
                array_keys($paramMap)
            );
            $query  = sprintf(
                'ALTER TABLE %s %s;',
                $db->quoteName('#__osdownloads_documents'),
                join(',', $fields)
            );

            $db->setQuery($query)->execute();
        }
    }

    /**
     * Detect legacy settings and fix the category view params. If legacy data
     * is found, warn the user. We call legacy data, the param category_id with
     * multiple values, in the Categories view. Represented issues for SEF
     * URLs, so we refactored allowing only one category.
     *
     * @return void
     * @throws Exception
     */
    protected function fixCategoryViewParams()
    {
        $db  = JFactory::getDbo();
        $app = JFactory::getApplication();

        // Look for menu items for Category view
        $query    = $db->getQuery(true)
            ->select(
                array(
                    'id',
                    'params',
                )
            )
            ->from('#__menu')
            ->where('link = ' . $db->quote('index.php?option=com_osdownloads&view=categories'));
        $menuList = $db->setQuery($query)->loadObjectList();

        if (!empty($menuList)) {
            foreach ($menuList as $menu) {
                $params = json_decode($menu->params);

                // Does it have the old param and multiple categories selected?
                if (isset($params->category_id) && is_array($params->category_id) && !empty($params->category_id)) {
                    // Get the first category for the new param. If empty, use 0, the root category
                    $id = (int)$params->category_id[0];

                    unset($params->category_id);

                    // Update the link adding the selected category
                    $link = 'index.php?option=com_osdownloads&view=categories&id=' . $id;

                    $query = $db->getQuery(true)
                        ->update('#__menu')
                        ->where('id = ' . (int)$menu->id)
                        ->set('link = ' . $db->quote($link))
                        ->set('params = ' . $db->quote(json_encode($params)));
                    $db->setQuery($query)->execute();

                    $app->enqueueMessage(
                        sprintf(
                            JText::_('Only one category is allowed for the OSDOwnloads Categories view. The params for menu item %s were upgraded.'),
                            $menu->id
                        ),
                        'warning'
                    );
                }
            }
        }
    }
}
